@@ -1,46 +1,21 @@
 package Collision::2D::Collision;
-use Mouse;
+
+use strict;
+use warnings;
+use Carp qw/carp croak confess/;
+require DynaLoader;
+our @ISA = qw(DynaLoader);
+bootstrap Collision::2D::Collision;
 
 #this might be of use for calculating bounce vectors based on axes of collision. 
 # http://www.members.shaw.ca/mathematica/ahabTutorials/2dCollision.html
 
-has 'axis' => (
-   is => 'ro',
-   #isa => 
-);
-
-has 'time' => (
-   is => 'ro',
-   isa => 'Num',
-   default=> 0,
-);
-has 'ent1' => (
-   isa => 'Collision::2D::Entity',
-   is => 'ro',
-);
-has 'ent2' => (
-   isa => 'Collision::2D::Entity',
-   is => 'ro',
-);
-
-sub maxis{
-   my $self = shift;
-   my $axis = $self->axis;
-   return unless $axis;
-   return $axis if ref $axis eq 'ARRAY';
-   if ($axis eq 'x'){
-      if ($self->ent1->relative_xv > 0){
-         return [1,0];
-      }
-      return [-1,0];
-   }
-   #$axis eq 'y'
-   if ($self->ent1->relative_yv > 0){
-      return [0,1];
-   }
-   return [0,-1];
+sub new{
+   my ($package,%params) = @_;
+   return __PACKAGE__->_new (@params{qw/ent1 ent2 time axis/})
 }
 
+
 sub does_mario_defeat_goomba{}
 
 #unless 'elasticity' is a param, assume it's totally elastic
@@ -56,13 +31,19 @@ use Math::Trig qw/acos/;
 sub bounce_vector{ 
    my ($self,%params) = @_;
    my $elasticity = $params{elasticity} // 1;
-   my $axis = $self->maxis;
+   my $axis = $self->vaxis;
+   unless ($axis){
+      confess 'no bounce vector because no axis.';
+      return [0,0];
+   }
    
    my $axis_len = sqrt($axis->[0]**2 + $axis->[1]**2);
    my $rxv = $self->ent1->relative_xv;
    my $ryv = $self->ent1->relative_yv;
    my $rv_len = sqrt($rxv**2 + $ryv**2);
    my $dot = $rxv*$axis->[0] + $ryv*$axis->[1];
+ #  warn $rv_len;
+   return [0,0] unless $rv_len;
    my $angle = acos($dot / ($axis_len * $rv_len));
    
    my $axis_scalar = $rv_len * cos($angle) / $axis_len;
@@ -78,10 +59,23 @@ sub bounce_vector{
    return [$r_bounce_xv + $self->ent2->xv, $r_bounce_yv + $self->ent2->yv]
 }
 
-
-no Mouse;
-__PACKAGE__->meta->make_immutable;
-1
+sub invert{
+   my $self = shift;
+   my $axis = $self->axis;
+   if (ref($axis) eq 'ARRAY'){
+      $axis = [-$axis->[0], -$axis->[1]]
+   }
+   else{ #x or y
+      $self->ent2->normalize($self->ent1);
+   }
+   return Collision::2D::Collision->new(
+      ent1=>$self->ent2,
+      ent2=>$self->ent1,
+      time=>$self->time,
+      axis => $axis,
+   )
+   
+}
 
 __END__
 =head1 NAME
@@ -113,7 +107,7 @@ This vector will not be normal (normal means of length 1).
 L<Collision::2D::normalize_vec($v)|Collision::2D/normalize_vec>
 is provided for that purpose.
 
-=item maxis
+=item vaxis
 
 Again, the axis of collision. If you call this, it will always return the mathematical
 form [$x,$y]. If the axis existed as 'x' or 'y', it is translated to [$x,$y].
@@ -131,3 +125,16 @@ dynamic_collision doesn't preserve the order of your entities.
 
 =back
 
+=head1 METHODS
+
+=over
+
+=item bounce_vector
+
+=item invert
+
+my $other_collision = $self->invert();
+
+This returns the inverse of this collision. That is, the time remains,
+but ent1 and ent2 are swapped, and the axis is inversed. This does not effect $self.
+
